--[[
-------------------------------------------------------------------------------
-- ChatStamp, by Ayantir
-------------------------------------------------------------------------------
This software is under : CreativeCommons CC BY-NC-SA 4.0
Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)

You are free to:

    Share — copy and redistribute the material in any medium or format
    Adapt — remix, transform, and build upon the material
    The licensor cannot revoke these freedoms as long as you follow the license terms.


Under the following terms:

    Attribution — You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
    NonCommercial — You may not use the material for commercial purposes.
    ShareAlike — If you remix, transform, or build upon the material, you must distribute your contributions under the same license as the original.
    No additional restrictions — You may not apply legal terms or technological measures that legally restrict others from doing anything the license permits.


Please read full licence at : 
http://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
]]

local defaults = 
{
	military = true,			-- true = 24 hour clock, false = 12 hour clock
	systemcs = true,			-- true = add timestamps to system messages
	showsecs = false,			-- true = seconds will be added.
	dontfade = true,			-- true = text won't fade
	addrgb = false,			-- true = use custom timestamp color
	clockrgb = "|cffffff",	-- sets the color of the timestamp
}

local db
local ADDON_NAME = "ChatStamp"
local ADDON_AUTHOR = "Ayantir & SnowmanDK"
local ADDON_VERSION = "5"
local ADDON_WEBSITE = "http://www.esoui.com/downloads/info674-ChatStamp.html"

local function AddChatStamp()
	
	local ourTime = GetSecondsSinceMidnight()
	local secsDigits = ourTime % 60
	
	ourTime = (ourTime - secsDigits) / 60
	local minutesDigits = ourTime % 60
	ourTime = (ourTime - minutesDigits) / 60

	local buf = {}
	buf[1] = ""
	buf[2] = ourTime
	buf[3] = ":"
	buf[4] = minutesDigits < 10 and "0" or ""
	buf[5] = minutesDigits
	
	if db.showsecs then
		buf[6] = ":"
		buf[7] = secsDigits < 10 and "0" or ""
		buf[8] = secsDigits
	end
	
	if db.military then
		if ourTime < 10 then
			buf[1] = 0
		end
	else
		buf[#buf+1] = ourTime > 11 and " PM" or " AM"
		ourTime = ourTime % 12
		buf[2] = ourTime == 0 and 12 or ourTime
	end
	
	local chattime = table.concat(buf, "")
	
	if db.addrgb then
		chattime = db.clockrgb.. "[" .. chattime .. "]|r "
	else
		chattime = "["..chattime.."] "
	end
	
	return chattime
		
end

local function OnAddonLoaded(_, addOnName)

	local function getColour(colourString)
		if not string.match(colourString, "|c") then colourString = "|c"..colourString end
		local r=tonumber(string.sub(colourString, 3, 4), 16)
		local g=tonumber(string.sub(colourString, 5, 6), 16)
		local b=tonumber(string.sub(colourString, 7, 8), 16)
		return r/255, g/255, b/255, 1
	end

	local function defaultColour(colourString)
		if not string.match(colourString, "|c") then colourString = "|c"..colourString end
		local r=tonumber(string.sub(colourString, 3, 4), 16)
		local g=tonumber(string.sub(colourString, 5, 6), 16)
		local b=tonumber(string.sub(colourString, 7, 8), 16)
		return {["r"] = r/255, ["g"] = g/255, ["b"] = b/255, ["a"] = 1}
	end

	-- Convert a decimal number in [0,255] to a hexadecimal number.
	local function d2h(n)
		local str = "0123456789abcdef"
		local l = string.sub(str, math.floor(n/16)+1, math.floor(n/16)+1)
		local r = string.sub(str, n%16 + 1, n%16 + 1)
		return l..r
	end

	-- Turn a ([0,1])^3 RGB colour to "|cABCDEF" form.
	local function makeColour(r, g, b)
		r=math.floor(255*r)
		g=math.floor(255*g)
		b=math.floor(255*b)
		return "|c"..d2h(r)..d2h(g)..d2h(b)
	end

	if addOnName ~= ADDON_NAME then return end
	if pChat then return end
	
	db = ZO_SavedVars:New("ChatStamp_Settings", 1, nil, defaults)
	
	local LAM2 = LibStub:GetLibrary("LibAddonMenu-2.0")
	local LC = LibStub:GetLibrary("libChat-1.0")
	
	LC:registerAppendTextBeforeAll(AddChatStamp, ADDON_NAME)
	
	local panelData = 
	{
		type = "panel",
		name = ADDON_NAME,
		displayName = ADDON_NAME,
		author = ADDON_AUTHOR,
		version = ADDON_VERSION,
		website = ADDON_WEBSITE,
		registerForRefresh = true,
		registerForDefaults = true,
	}
	
	local optionsData = 
	{		
		{
			type = "checkbox",
			name = GetString(CHATSTAMP_24H),
			width = "full",
			tooltip = GetString(CHATSTAMP_24H_DESC),
			getFunc = function() return db.military end,
			setFunc = function(value) db.military = value end,
			default = defaults.military
		},
		{
			type = "checkbox",
			name = GetString(CHATSTAMP_SECONDS),
			width = "full",
			tooltip = GetString(CHATSTAMP_SECONDS_DESC),
			getFunc = function() return db.showsecs end,
			setFunc = function(value) db.showsecs = value end,
			default = defaults.showsecs
		},
		{
			type = "checkbox",
			name = GetString(CHATSTAMP_CUSTOM),
			width = "full",
			tooltip = GetString(CHATSTAMP_CUSTOM_DESC),
			getFunc = function() return db.addrgb end,
			setFunc = function(value) db.addrgb = value end,
			default = defaults.addrgb
		},
		{
			type = "colorpicker",
			name = GetString(CHATSTAMP_COLOR),
			width = "full",
			tooltip = GetString(CHATSTAMP_COLOR_DESC),
			getFunc = function() return getColour(db.clockrgb) end,
			setFunc = function(r, g, b) db.clockrgb = makeColour(r, g, b) end,
			default = defaultColour(db.clockrgb),
			disabled = function() return not db.addrgb end,
		}, 
	}
	
	local myPanel = LAM2:RegisterAddonPanel(ADDON_NAME .. "LAM2Options", panelData)
	LAM2:RegisterOptionControls(ADDON_NAME .. "LAM2Options", optionsData)
	
	EVENT_MANAGER:UnregisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED)
	
end

EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddonLoaded)